@@ -6,7 +6,7 @@ import itertools
 import warnings
 import numpy as np
 from shapely import wkt
-from shapely.geometry import Point, Polygon, MultiPoint, MultiPolygon, LineString, MultiLineString, LinearRing, mapping
+from shapely.geometry import Point, Polygon, MultiPoint, MultiPolygon, LineString, MultiLineString, LinearRing, mapping, shape
 from shapely.ops import transform, linemerge, unary_union, polygonize
 from shapely.affinity import translate
 import pandas as pd
@@ -36,6 +36,7 @@ from scipy.interpolate import interp1d
 import jenkspy
 
 import zipfile
+import tarfile
 import utm
 import numbers
 import osmnx as osm
@@ -206,9 +207,11 @@ class LaharFlowData(object):
                     
             self.files=[]
             self.txt_files=[]
+            self.tar_files=[]
             
             self.get_files()
             self.get_txt_files()
+            self.get_tar_files()
             self.get_tiff_files()
 
             self.clatlong = [self.clat,self.clong]
@@ -223,6 +226,8 @@ class LaharFlowData(object):
                 self._get_parameters_from_runinfo()
             self._get_max_depth()
             self._get_max_speed()
+
+            self.rasters=dict()
     
     def _get_parameters_from_inputfile(self):
         if self.zipped:
@@ -380,6 +385,10 @@ class LaharFlowData(object):
         else:
             self.files = os.listdir(self.dir)
     
+    def get_tar_files(self):
+        self.tar_files = [f for f in self.files if f.endswith('.tar.gz')]
+        self.tar_files.sort()
+
     def get_txt_files(self):
         self.txt_files = [f for f in self.files if f.endswith('.txt')]
         self.txt_files.sort()
@@ -417,8 +426,11 @@ class LaharFlowData(object):
             if fileIn not in self.files:
                 raise ValueError('Result file {file} is not in directory {dir}'.format(file=fileIn, dir=self.dir))
         elif isinstance(fileIn, int):
-            fileIn = '{0:06d}.txt'.format(fileIn)
-            if fileIn not in self.files:
+            if '{0:06d}.txt'.format(fileIn) in self.files:
+                fileIn = '{0:06d}.txt'.format(fileIn)
+            elif '{0:06d}.txt.tar.gz'.format(fileIn) in self.files:
+                fileIn = '{0:06d}.txt.tar.gz'.format(fileIn)
+            else:
                 raise ValueError('Result file {file} is not in directory {dir}'.format(file=fileIn, dir=self.dir))
         else:
             raise RuntimeError("Result must be either the name of file in directory {dirc}, or an integer".format(dirc=self.dir))
@@ -442,24 +454,36 @@ class LaharFlowData(object):
         if variable not in _LFoutput_shortnames:
             raise RuntimeError('variable {var} not recognized'.format(var=variable))
 
+    def _extract_file(self, filename):
+        if self.zipped:
+            LaharFlowFilePath = _path_of_file_in_zip(filename, self._zipref)[0]
+            LaharFlowFile = self._zipref.extract(LaharFlowFilePath, path=self.dir)
+            ExtractPath = os.path.join(self.dir, os.path.split(LaharFlowFilePath)[0])
+        elif filename in self.txt_files:
+            LaharFlowFile = os.path.join(self.dir, filename)
+            ExtractPath = self.dir
+        
+        if (os.path.split(LaharFlowFile)[1] in self.tar_files) and (os.path.split(LaharFlowFile)[1] not in self.txt_files):
+            tar = tarfile.open(LaharFlowFile, mode='r')
+            tar.extractall(path=ExtractPath)
+            tar.close()
+            self.get_txt_files()
+            LaharFlowFile = '.'.join(LaharFlowFile.split('.')[:-2])
+        return LaharFlowFile
+
     def _get_data_column_names(self, filename):
         filename = self._valid_input_file(filename)
-        if self.zipped:
-            LaharFlowFilePath = _path_of_file_in_zip(filename,self._zipref)[0]
-            LaharFlowFile = self._zipref.extract(LaharFlowFilePath,path=self.dir)
-        else:
-            LaharFlowFile = os.path.join(self.dir,filename)
+        LaharFlowFile = self._extract_file(filename)
+        
         cols = csv_active_cols(LaharFlowFile)
         return cols
 
     def _get_data(self, filename):
 
         filename = self._valid_input_file(filename)
-        if self.zipped:
-            LaharFlowFilePath = _path_of_file_in_zip(filename, self._zipref)[0]
-            LaharFlowFile = self._zipref.extract(LaharFlowFilePath, path=self.dir)
-        else:
-            LaharFlowFile = os.path.join(self.dir, filename)
+
+        LaharFlowFile = self._extract_file(filename)
+        
         try:
             cols = csv_active_cols(LaharFlowFile)
             data = np.genfromtxt(LaharFlowFile, delimiter=',', names=True, usecols=cols)
@@ -467,23 +491,22 @@ class LaharFlowData(object):
         except:
             return None
     
-    def raster(self, filename, full_domain=False, nodata=np.nan, masked=True, crs=None):
+    def raster(self, filename, var=None, full_domain=False, nodata=np.nan, masked=True, crs=None, store_local=True):
         filetype = self._file_type(filename)
         
         filename = self._valid_input_file(filename)
         
-        if self.zipped:
-            LaharFlowFilePath = _path_of_file_in_zip(filename, self._zipref)[0]
-            LaharFlowFile = self._zipref.extract(LaharFlowFilePath, path=self.dir)
-        else:
-            LaharFlowFile = os.path.join(self.dir, filename)
+        LaharFlowFile = self._extract_file(filename)
         
         cols = csv_active_cols(LaharFlowFile)
         data = np.genfromtxt(LaharFlowFile, delimiter=',', names=True, usecols=cols)
         
         extra_cols = ['tile', 'x_distance', 'y_distance', 'latitude', 'longitude']
-        data_cols = [x for x in data.dtype.names 
-                        if x not in extra_cols]
+        if var is None:
+            data_cols = [x for x in data.dtype.names 
+                            if x not in extra_cols]
+        else:
+            data_cols = [x for x in var]
         
         x = data['x_distance']
         y = data['y_distance']
@@ -523,7 +546,7 @@ class LaharFlowData(object):
             ds[col].attrs["units"] = fmt['units']
             ds[col].attrs["symbol"] = fmt['symbol']
         
-        if filetype=='snapshot':
+        if filetype=='snapshot' and 'concentration' in data_cols:
             ds['concentration'] = ds['mass_of_solids']
             ds['concentration'] = np.divide(ds['mass_of_solids'], ds['mass_per_unit_area'], where=ds['mass_per_unit_area'].where(ds['mass_per_unit_area'].data>1e-8))
             fmt = _get_LF_output_fmt('concentration')
@@ -540,6 +563,9 @@ class LaharFlowData(object):
         if crs is not None:
             ds = ds.rio.reproject(crs)
         
+        if store_local:
+            self.rasters[os.path.splitext(filename)] = ds
+
         return ds
 
     def _raster_flatten(raster):
@@ -548,12 +574,8 @@ class LaharFlowData(object):
 
     def _get_data_column(self, filename, data_col):
 
-        if self.zipped:
-            dataFilePath = _path_of_file_in_zip(filename,self._zipref)[0]
-            dataFile = self._zipref.extract(dataFilePath,path=self.dir)
-        else:
-            dataFile = os.path.join(self.dir, filename)
-        
+        dataFile = self._extract_file(filename)
+
         try:
             data = np.genfromtxt(dataFile,delimiter=',',comments='%',skip_header=1)
             x = data[:,1]
@@ -594,11 +616,9 @@ class LaharFlowData(object):
 
     def _get_max_bed_evolution(self):
         last_outfile = self.get_snapshot_files()[-1]
-        if self.zipped:
-            dataFilePath = _path_of_file_in_zip(last_outfile,self._zipref)[0]
-            dataFile = self._zipref.extract(dataFilePath,path=self.dir)
-        else:
-            dataFile = os.path.join(self.dir, last_outfile)
+        
+        dataFile = self._extract_file(last_outfile)
+
         if not os.path.isfile(dataFile):
             MaxErosion = np.nan
             MaxDeposit = np.nan
@@ -615,7 +635,13 @@ class LaharFlowData(object):
     
     def max_bed_evolution(self, vmin=None, vmax=None):
         last_outfile = self.get_snapshot_files()[-1]
-        data = self.raster(last_outfile)
+        if os.path.splitext(last_outfile) not in self.rasters.keys():
+            data = self.raster(last_outfile)
+        else:
+            data = self.rasters[last_outfile]
+            if 'elevation_change' not in list(data.keys()):
+                data = self.raster(last_outfile)
+            
         bt = data['elevation_change']
         
         if vmin is not None:
@@ -634,6 +660,12 @@ class LaharFlowData(object):
     
     def max_depth(self, vmin=None):
 
+        if 'MaxHeights' not in self.rasters.keys():
+            data = self.raster('MaxHeights.txt')
+        else:
+            data = self.rasters['MaxHeights']
+            if 'maximum_depth' not in list(data.keys()):
+                data = self.raster('MaxHeights.txt')
         data = self.raster('MaxHeights.txt')
         maxh = data['maximum_depth']
 
@@ -685,12 +717,10 @@ class LaharFlowData(object):
     def snapshot_data(self,fnum):
         assert isinstance(fnum,int), 'in getSnapshotData, input must be an integer'
         fnum = '{0:06d}.txt'.format(fnum)
-        assert(fnum in self.files), 'Results file {file} is not in directory {dir}'.format(file=fnum,dir=self.dir)
-        if self.zipped:
-            LaharFlowFilePath = _path_of_file_in_zip(fnum,self._zipref)[0]
-            LaharFlowFile = self._zipref.extract(LaharFlowFilePath,path=self.dir)
-        else:
-            LaharFlowFile = os.path.join(self.dir,fnum)
+        assert( any([c in self.files for c in ('{0:06d}.txt'.format(fnum), '{0:06d}.txt.tar.gz'.format(fnum))] )), 'Results file {file} is not in directory {dir}'.format(file=fnum,dir=self.dir)
+
+        LaharFlowFile = self._extract_file(fnum)
+
         cols = csv_active_cols(LaharFlowFile)
         data = np.genfromtxt(LaharFlowFile,delimiter=',',names=True,usecols=cols)
         return data
@@ -714,6 +744,7 @@ class LaharFlowData(object):
     def _raster_contour(self, raster, name, cntrs):
 
         data = raster.data
+        crs = raster.rio.crs
         x = raster.x.data
         y = raster.y.data
 
@@ -721,42 +752,44 @@ class LaharFlowData(object):
         x0 += self.utm[0]
         y0 += self.utm[1]
         
-        fx = interp1d(np.arange(0,len(x0)),x0)
-        fy = interp1d(np.arange(0,len(y0)),y0)
-
-        # fx = interp1d(np.arange(0,len(x)),x)
-        # fy = interp1d(np.arange(0,len(y)),y)
+        fx = interp1d(np.arange(0,len(x)),x)
+        fy = interp1d(np.arange(0,len(y)),y)
 
+        
         for kk, this_cntr in enumerate(cntrs):
             
-            C = find_contours(data, this_cntr)
+            new_set = True
             
+            cdata = np.zeros_like(data)
+            cdata[data>=this_cntr] = 1
+            C = find_contours(cdata, 0.5)
+
             if len(C)==0:
                 pass
             for jj, p in enumerate(C):
-                p[:,0] = fy(p[:, 0])
-                p[:,1] = fx(p[:, 1])
+                if len(p)>2:
+                    p[:,0] = fy(p[:, 0])
+                    p[:,1] = fx(p[:, 1])
+                    
+                    p[:, [0, 1]] = p[:, [1, 0]]
                 
-                p[:, [0, 1]] = p[:, [1, 0]]
+                    thisPoly = Polygon(p).buffer(0)
                 
-                thisPoly = Polygon(p).buffer(0)
-                if not thisPoly.is_empty:
-                    if jj==0:
-                        geom = thisPoly
-                        g_tmp = gpd.GeoDataFrame(columns=['contour', 'name', 'geometry'],
-                                                 crs=self.utmCode)
-                        g_tmp.loc[0, 'contour'] = this_cntr
-                        g_tmp.loc[0, 'geometry'] = geom
-                        g_tmp.loc[0, 'name'] = name
-                    else:
-                        if g_tmp.loc[0, 'geometry'].contains(thisPoly):
-                            geom = g_tmp.loc[0, 'geometry'].difference(thisPoly)
+                    if not thisPoly.is_empty:
+                        if new_set:
+                            new_set = False
+                            geom = thisPoly
+                            
                         else:
-                            try:
-                                geom = g_tmp.loc[0, 'geometry'].union(thisPoly)
-                            except:
-                                print('Error processing polygon contour -- skipping.  Better check the result!')
-                        g_tmp.loc[[0], 'geometry'] = gpd.GeoSeries([geom])
+                            geom1 = thisPoly
+                            
+                            geom = geom.symmetric_difference(geom1)
+                            
+            g_tmp = gpd.GeoDataFrame(columns=['contour', 'name', 'geometry'],
+                                 crs=self.utmCode)
+            g_tmp.loc[0, 'contour'] = this_cntr
+            g_tmp.loc[[0], 'geometry'] = gpd.GeoSeries(geom)
+            g_tmp.loc[0, 'name'] = name
                 
             if kk==0:
                 g = gpd.GeoDataFrame(g_tmp).set_geometry('geometry')
@@ -764,6 +797,7 @@ class LaharFlowData(object):
                 g = gpd.GeoDataFrame(pd.concat([g, g_tmp], ignore_index=True))
             
         g['contour'] = g['contour'].astype('float64')
+        g = g.set_crs(crs)
         
         return g
 
@@ -772,12 +806,12 @@ class LaharFlowData(object):
         maxh = self.max_depth(vmin=vmin)
 
         if maxh is not None:
-            maxh = maxh['maximum_depth']
+            #maxh = maxh['maximum_depth']
             if type(cntrs)==np.ndarray:
                 cntrs = cntrs.tolist()
             else:
                 if cntrs=='Jenks':
-                    cntrs = jenkspy.jenks_breaks(_raster_flatten(maxh), nb_class=10)
+                    cntrs = jenkspy.jenks_breaks(self._raster_flatten(maxh), nb_class=10)
                     cntrs = [c for c in cntrs if c>=vmin]
                     cntrs = cntrs[:-1]
                 if type(cntrs) is not list:
@@ -800,7 +834,7 @@ class LaharFlowData(object):
                 cntrs = cntrs.tolist()
             else:
                 if cntrs=='Jenks':
-                    cntrs = jenkspy.jenks_breaks(_raster_flatten(maxspd), nb_class=10)
+                    cntrs = jenkspy.jenks_breaks(self._raster_flatten(maxspd), nb_class=10)
                     cntrs = [c for c in cntrs if c>=vmin]
                     cntrs = cntrs[:-1]
                 if type(cntrs) is not list:
@@ -822,7 +856,7 @@ class LaharFlowData(object):
                 cntrs = cntrs.tolist()
             else:
                 if cntrs=='Jenks':
-                    cntrs = jenkspy.jenks_breaks(_raster_flatten(itime), nb_class=10)
+                    cntrs = jenkspy.jenks_breaks(self._raster_flatten(itime), nb_class=10)
                     cntrs = [c for c in cntrs if c>=vmin]
                     cntrs = cntrs[:-1]
                 if type(cntrs) is not list:
@@ -836,6 +870,48 @@ class LaharFlowData(object):
         else:
             return None
     
+    def POI_value(self, filename, x, y, var=None, full_domain=False, nodata=np.nan, fill_value=0, interp_method='linear'):
+        
+        if var is not None:
+            var = [var]
+
+        if interp_method not in ['nearest', 'linear'] and fill_value is None:
+            RuntimeError("fill_value must be specified unless using 'nearest' or 'linear' interp_method.")
+
+        data = self.raster(filename, var=var, full_domain=full_domain, nodata=np.nan, masked=True, crs=None)
+
+        if fill_value is not None:
+            data = data.fillna(fill_value)
+        
+        value = data.interp(x=xA, y=yA, method=interp_method)
+
+        val = dict()
+        vars = var if var is not None else list(data.keys())
+        for v in vars:
+            val[v] = value[v].values
+        
+        return val
+    
+    def transect_value(self, filename, x, y, var=None, full_domain=False, nodata=np.nan, fill_value=0, interp_method='linear'):
+        
+        if var is not None:
+            var = [var]
+
+        xA = xa.DataArray(x, dims="z")
+        yA = xa.DataArray(y, dims="z")
+
+        if interp_method not in ['nearest', 'linear'] and fill_value is None:
+            RuntimeError("fill_value must be specified unless using 'nearest' or 'linear' interp_method.")
+
+        data = self.raster(filename, var=var, full_domain=full_domain, nodata=np.nan, masked=True, crs=None)
+
+        if fill_value is not None:
+            data = data.fillna(fill_value)
+        
+        value = data.interp(x=xA, y=yA, method=interp_method)
+
+        return value
+
     def save_gtiff(self, fileIn, fileOut=None, crs=None, **kwargs):
 
         fileIn = self._valid_input_file(fileIn)
@@ -908,4 +984,4 @@ class LaharFlowData(object):
         ax.get_xaxis().set_visible(False) 
         ax.get_yaxis().set_visible(False)
 
-        return ax, im, cbar
\ No newline at end of file
+        return ax, im, cbar
